import { world } from "@minecraft/server";

function generateUUID() {
    return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function(c) {
        const r = Math.random() * 16 | 0;
        const v = c === 'x' ? r : (r & 0x3 | 0x8);
        return v.toString(16);
    });
}

const dimension = world.getDimension("minecraft:overworld");

class SpellBookDatabase{
    constructor(identifier){
        this.name = "Book";
        this.max_spell = 1;
        this.compatible_attributes = [
            "creation",
            "restoration",
            "conjuration",
            "destruction",
            "transmutation",
            "manipulation",
            "nature",
            "fire", 
            "void", 
            "dark",
            "water",
            "thunder",
            "wind",
            "light",
            "ice"
        ];
        this.incompatible_attributes = [];
        this.compatible_types = [
            "create",
            "action",
            "summon",
            "enchance",
            "combat",
            "curse"
        ];
        this.grade = [ 0, 0 ];
    }

    addPredefinedSpell(spell_name){
        if(this.predefine){
            this.predefine.spell.push(spell_name);
            this.predefine.locked_spell.push((this.predefine.spell.length - 1));
        }else{
            this.predefine = {
                locked_spell: [ 0 ],
                spell: [
                    spell_name
                ]
            }
        }
    }

    getJSON(){
        return {
            path: "",
            name: this.name,
            max_spell: this.max_spell,
            compatible_attributes: this.compatible_attributes,
            compatible_types: this.compatible_types,
            incompatible_attributes: this.incompatible_attributes,
            grade: this.grades,
            predefine: this.predefine
        }
    }
}

class CastingWeaponDatabase{
    constructor(identifier){
        this.fast_swap = false;
        this.soul_efficiency = 1.0;
        this.cast_efficiency = 1.0;
    }

    getJSON(){
        let data = {
            cast_efficiency: this.cast_efficiency,
            soul_efficiency: this.soul_efficiency
        }
        if(this.fast_swap) data.fast_swap = {}

        return data;
    }
}

class PredefineAttributesDatabase{
    constructor(identifier){
        this.identifier;
    }
    
    addAttribute(attribute, point){
        this[attribute] = point;
    }
}

export const GradeList = {
	Basic: 0,
	Intermediate: 1,
	Advance: 2,
	Expert: 3,
	Master: 4
}

export const MagicTypes = {
    Create: "create",
    Action: "action",
    Summon: "summon",
    Enchance: "enchance",
    Combat: "combat",
    Curse: "curse"
}

export class MagicLibrary{
    constructor(){ 
        this.id = generateUUID();
        this.SendDatabase = {
            spell_book: {},
            casting_weapon: {},
            magic_attributes: {},
            predefine_attributes: {},
            magic: {}
        }
    }

    addSpellBook(id){
        let data = new SpellBookDatabase(id);
        this.SendDatabase.spell_book[id] = data;
        return data;
    }

    addCastingWeapon(id){
        let data = new CastingWeaponDatabase(id);
        this.SendDatabase.casting_weapon[id] = data;
        return data;
    }

    addAttributePoint(id, points){
        this.SendDatabase.magic_attributes[id] = points;
    }

    addPredefineAttributes(id){
        let data = new PredefineAttributesDatabase(id);
        this.SendDatabase.predefine_attributes[id] = data;
        return data;
    }

    send(){
        console.warn(JSON.stringify(this.SendDatabase))

        for(let split_data of JSON.stringify(this.SendDatabase).match(/.{1,255}/g)){
            dimension.runCommand("scriptevent amw_load_database:" + this.id + " " + split_data);
        }
    }
}